class Jira
  attr_accessor :url
  attr_accessor :user
  attr_accessor :db
  attr_accessor :all

  def initialize (config)
    @url = config['jira']['host']
    @user = config['jira']['user']
    @db = Influx.new(config['influxdb']['host'], config['jira']['dbname'], config['influxdb']['port'], config['influxdb']['user'], config['influxdb']['password'])
    jira_options = {
      :site         => @url,
      :username     => @user,
      :password     => config['jira']['password'],
      :context_path => '',
      :auth_type    => :basic,
      :read_timeout => 60,
    }
    @connection = JIRA::Client.new(jira_options)
    max_results_per_query = -1
    issues = []
    begin
      issues_v = @connection.Issue.jql('', max_results: max_results_per_query, start_at: issues.size)
      issues = issues + issues_v
    end until issues_v.empty?
    @all = issues.flatten.sort_by(&:id)
    return @connection
  end

  def owner_list()
    owners = []
    self.all.each do |issue|
      if issue.assignee != nil
        owners << issue.assignee.displayName
      end
    end
    owners.uniq!.sort!
  end

  def status_list()
    status_list = []
    self.all.each do |issue|
      status_list << issue.status.name
    end
    status_list.uniq!.sort!
  end

  def priority_list()
    priority_list = []
    self.all.each do |issue|
      if issue.priority != nil
        priority_list << issue.priority.name
      end
    end
    priority_list.uniq!.sort!
  end

  def project_list()
    project_list = []
    self.all.each do |issue|
      if issue.project != nil
        project_list << issue.project.name
      end
    end
    project_list.uniq!.sort!
  end

  def by_status(status)
    issues = self.all.select{|issue| issue.status != nil}.select{|issue| issue.status.name == status}
  end

  def by_owner(owner)
    issues = self.all.select{|issue| issue.assignee != nil}.select{|issue| issue.assignee.displayName == owner}
  end

  def by_priority(priority)
    issues = self.all.select{|issue| issue.priority != nil}.select{|issue| issue.priority.name == priority}
  end

  def by_project(project)
    issues = self.all.select{|issue| issue.project != nil}.select{|issue| issue.project.name == project}
  end

  def by_overdue()
    issues = self.all.select{|issue| issue.duedate != nil}.select{|issue| Time.parse(issue.duedate) < Time.now}.select{|issue| issue.status.name != "Done"}.select{|issue| issue.status.name != "Closed"}
  end

  def by_owner_nid(owner)
    issues = self.all.select{|issue| issue.assignee != nil}.select{|issue| issue.assignee.name == owner}
  end

  def by_owner_and_status(owner, status)
    issues = self.all.select{|issue| issue.assignee != nil}.select{|issue| issue.assignee.displayName == owner}.select{|issue| issue.status.name == status}
  end

  def by_owner_and_priority(owner, priority)
    issues = self.all.select{|issue| issue.assignee != nil}.select{|issue| issue.assignee.displayName == owner}.select{|issue| issue.priority.name == priority}
  end

  def own()
    self.by_owner_nid(self.user)
  end
end
